"""
ProjPack CLI

Command-line interface for ProjPack project bundling tool.
Provides user-friendly commands for packing projects and managing ignore files.
"""

import argparse
import sys
from pathlib import Path
from typing import Optional

try:
    from .core import ProjectPacker
    from .ignore_parser import create_default_ppignore
except ImportError:
    # Handle standalone execution
    from core import ProjectPacker
    from ignore_parser import create_default_ppignore


def format_size(size_bytes: int) -> str:
    """Format file size in human-readable format."""
    if size_bytes < 1024:
        return f"{size_bytes} B"
    elif size_bytes < 1024 * 1024:
        return f"{size_bytes / 1024:.1f} KB"
    elif size_bytes < 1024 * 1024 * 1024:
        return f"{size_bytes / (1024 * 1024):.1f} MB"
    else:
        return f"{size_bytes / (1024 * 1024 * 1024):.1f} GB"


def pack_command(args):
    """Handle the pack command."""
    try:
        # Validate input directory
        if not Path(args.input).exists():
            print(f"Error: Input directory '{args.input}' does not exist.")
            return 1

        if not Path(args.input).is_dir():
            print(f"Error: '{args.input}' is not a directory.")
            return 1

        # Generate default output filename if not provided
        if not args.output:
            project_name = Path(args.input).resolve().name
            args.output = f"{project_name}_bundle.txt"

        # Check if output file already exists
        if Path(args.output).exists() and not args.force:
            response = input(f"Output file '{args.output}' already exists. Overwrite? (y/N): ")
            if response.lower() != 'y':
                print("Operation cancelled.")
                return 0

        # Create packer instance
        packer = ProjectPacker(
            project_root=args.input,
            ignore_file=args.ignore_file,
            max_file_size=args.max_size * 1024 * 1024,  # Convert MB to bytes
            include_stats=not args.no_stats
        )

        print(f"Packing project: {Path(args.input).resolve()}")

        # Pack the project
        bundle_content = packer.pack_project(
            output_file=args.output,
            verbose=args.verbose
        )

        # Get summary
        summary = packer.get_project_summary()

        # Print results
        print(f"\n✓ Bundle created successfully!")
        print(f"  Output: {Path(args.output).resolve()}")
        print(f"  Files processed: {summary['processed_files']}")
        print(f"  Bundle size: {format_size(len(bundle_content.encode('utf-8')))}")

        if args.verbose or summary['ignored_files'] > 0:
            print(f"\n  Files ignored: {summary['ignored_files']}")
            print(f"  Binary files skipped: {summary['binary_files']}")
            print(f"  Large files skipped: {summary['large_files']}")
            if summary['error_files'] > 0:
                print(f"  Files with errors: {summary['error_files']}")

        return 0

    except Exception as e:
        print(f"Error: {e}")
        if args.verbose:
            import traceback
            traceback.print_exc()
        return 1


def scan_command(args):
    """Handle the scan command (analyze without creating bundle)."""
    try:
        # Validate input directory
        if not Path(args.input).exists():
            print(f"Error: Input directory '{args.input}' does not exist.")
            return 1

        # Create packer instance
        packer = ProjectPacker(
            project_root=args.input,
            ignore_file=args.ignore_file,
            max_file_size=args.max_size * 1024 * 1024
        )

        print(f"Scanning project: {Path(args.input).resolve()}")

        # Scan the project
        files, stats = packer.scan_project(verbose=args.verbose)
        summary = packer.get_project_summary()

        # Print detailed results
        print(f"\n📊 Scan Results:")
        print(f"  Project root: {summary['project_root']}")
        print(f"  Directories scanned: {summary['directories_scanned']}")
        print(f"  Total files found: {summary['total_files']}")
        print(f"  Files to process: {summary['processed_files']}")
        print(f"  Total content size: {summary['total_size_formatted']}")
        print(f"  Processing time: {summary['processing_time']:.2f}s")

        if summary['ignored_files'] > 0 or summary['binary_files'] > 0 or summary['large_files'] > 0:
            print(f"\n📋 Skipped files:")
            print(f"  Ignored (by patterns): {summary['ignored_files']}")
            print(f"  Binary files: {summary['binary_files']}")
            print(f"  Large files (>{args.max_size}MB): {summary['large_files']}")
            if summary['error_files'] > 0:
                print(f"  Error files: {summary['error_files']}")

        # List files that would be processed
        if args.list_files:
            print(f"\n📁 Files to be processed:")
            for file_path in files:
                rel_path = file_path.relative_to(Path(args.input))
                file_size = file_path.stat().st_size
                print(f"  {rel_path} ({format_size(file_size)})")

        return 0

    except Exception as e:
        print(f"Error: {e}")
        if args.verbose:
            import traceback
            traceback.print_exc()
        return 1


def init_command(args):
    """Handle the init command (create default .ppignore)."""
    try:
        target_dir = args.directory or "."

        print(f"Creating default .ppignore file in: {Path(target_dir).resolve()}")
        create_default_ppignore(target_dir)

        return 0

    except Exception as e:
        print(f"Error: {e}")
        return 1


def create_parser():
    """Create and configure the argument parser."""
    parser = argparse.ArgumentParser(
        prog="projpack",
        description="Bundle project files into a single text file for AI analysis",
        epilog="Use 'projpack <command> --help' for command-specific help."
    )

    # Global options
    parser.add_argument(
        "--version",
        action="version",
        version="ProjPack 1.0.0"
    )

    subparsers = parser.add_subparsers(dest="command", help="Available commands")

    # Pack command
    pack_parser = subparsers.add_parser(
        "pack",
        help="Pack project files into a bundle",
        description="Bundle all relevant project files into a single text file"
    )
    pack_parser.add_argument(
        "input",
        help="Input project directory"
    )
    pack_parser.add_argument(
        "-o", "--output",
        help="Output bundle file (default: <project_name>_bundle.txt)"
    )
    pack_parser.add_argument(
        "-i", "--ignore-file",
        help="Custom ignore file path (default: .ppignore in project root)"
    )
    pack_parser.add_argument(
        "-s", "--max-size",
        type=int,
        default=10,
        help="Maximum file size in MB (default: 10)"
    )
    pack_parser.add_argument(
        "-v", "--verbose",
        action="store_true",
        help="Enable verbose output"
    )
    pack_parser.add_argument(
        "-f", "--force",
        action="store_true",
        help="Overwrite output file without confirmation"
    )
    pack_parser.add_argument(
        "--no-stats",
        action="store_true",
        help="Exclude statistics from bundle header"
    )
    pack_parser.set_defaults(func=pack_command)

    # Scan command
    scan_parser = subparsers.add_parser(
        "scan",
        help="Analyze project without creating bundle",
        description="Scan project and show what would be included in the bundle"
    )
    scan_parser.add_argument(
        "input",
        help="Input project directory"
    )
    scan_parser.add_argument(
        "-i", "--ignore-file",
        help="Custom ignore file path"
    )
    scan_parser.add_argument(
        "-s", "--max-size",
        type=int,
        default=10,
        help="Maximum file size in MB (default: 10)"
    )
    scan_parser.add_argument(
        "-v", "--verbose",
        action="store_true",
        help="Enable verbose output"
    )
    scan_parser.add_argument(
        "-l", "--list-files",
        action="store_true",
        help="List all files that would be processed"
    )
    scan_parser.set_defaults(func=scan_command)

    # Init command
    init_parser = subparsers.add_parser(
        "init",
        help="Create default .ppignore file",
        description="Generate a default .ppignore file with common patterns"
    )
    init_parser.add_argument(
        "directory",
        nargs="?",
        help="Directory to create .ppignore in (default: current directory)"
    )
    init_parser.set_defaults(func=init_command)

    return parser


def main():
    """Main CLI entry point."""
    parser = create_parser()

    # Parse arguments
    if len(sys.argv) == 1:
        parser.print_help()
        return 0

    args = parser.parse_args()

    # Execute command
    if hasattr(args, 'func'):
        return args.func(args)
    else:
        parser.print_help()
        return 0


def cli_entry_point():
    """Entry point for setup.py console_scripts."""
    try:
        exit_code = main()
        sys.exit(exit_code)
    except KeyboardInterrupt:
        print("\nOperation cancelled by user.")
        sys.exit(1)
    except Exception as e:
        print(f"Unexpected error: {e}")
        sys.exit(1)


if __name__ == "__main__":
    cli_entry_point()