"""
ProjPack Core

Main aggregation logic that orchestrates file collection and bundling.
Combines IgnoreParser and FileHandler to create comprehensive project bundles.
"""

import os
from datetime import datetime
from pathlib import Path
from typing import List, Dict, Optional, Tuple
from dataclasses import dataclass

try:
    from .ignore_parser import IgnoreParser
    from .file_handler import FileHandler
except ImportError:
    # Handle standalone execution
    from ignore_parser import IgnoreParser
    from file_handler import FileHandler


@dataclass
class ProjectStats:
    """Statistics about the processed project."""
    total_files: int = 0
    total_size: int = 0
    ignored_files: int = 0
    binary_files: int = 0
    large_files: int = 0
    error_files: int = 0
    directories_scanned: int = 0
    processing_time: float = 0.0


class ProjectPacker:
    """Main class for packing project files into a single bundle."""

    def __init__(self,
                 project_root: str = ".",
                 ignore_file: Optional[str] = None,
                 max_file_size: int = 10 * 1024 * 1024,
                 include_stats: bool = True):
        """
        Initialize the project packer.

        Args:
            project_root: Root directory of the project
            ignore_file: Path to custom ignore file
            max_file_size: Maximum file size to process (bytes)
            include_stats: Whether to include statistics in output
        """
        self.project_root = Path(project_root).resolve()
        self.include_stats = include_stats

        # Initialize components
        self.ignore_parser = IgnoreParser(
            ignore_file_path=ignore_file,
            project_root=str(self.project_root)
        )
        self.file_handler = FileHandler(max_file_size=max_file_size)

        # Statistics tracking
        self.stats = ProjectStats()

    def scan_project(self, verbose: bool = False) -> Tuple[List[Path], ProjectStats]:
        """
        Scan the project and collect all files to be processed.

        Args:
            verbose: Print detailed progress information

        Returns:
            Tuple of (files_to_process, stats)
        """
        start_time = datetime.now()

        if verbose:
            print(f"Scanning project: {self.project_root}")

        # Reset stats
        self.stats = ProjectStats()

        # Collect all files
        all_files = []
        ignored_paths = set()

        for root, dirs, files in os.walk(self.project_root):
            self.stats.directories_scanned += 1
            root_path = Path(root)

            # Get relative path for ignore checking
            try:
                rel_root = root_path.relative_to(self.project_root)
                rel_root_str = str(rel_root) if str(rel_root) != '.' else ''
            except ValueError:
                continue

            # Filter directories
            dirs_to_remove = []
            for d in dirs[:]:
                dir_path = os.path.join(rel_root_str, d) if rel_root_str else d
                if self.ignore_parser.should_ignore(dir_path, is_directory=True):
                    dirs_to_remove.append(d)
                    ignored_paths.add(dir_path)
                    if verbose:
                        print(f"  Ignoring directory: {dir_path}")

            for d in dirs_to_remove:
                dirs.remove(d)

            # Process files
            for f in files:
                file_path = root_path / f
                rel_file_path = os.path.join(rel_root_str, f) if rel_root_str else f

                self.stats.total_files += 1

                # Check ignore patterns
                if self.ignore_parser.should_ignore(rel_file_path, is_directory=False):
                    self.stats.ignored_files += 1
                    ignored_paths.add(rel_file_path)
                    if verbose:
                        print(f"  Ignoring file: {rel_file_path}")
                    continue

                # Check if file should be processed
                should_process, reason = self.file_handler.should_process_file(file_path)

                if should_process:
                    all_files.append(file_path)
                    self.stats.total_size += self.file_handler.get_file_size(file_path)
                else:
                    if "Binary file" in reason:
                        self.stats.binary_files += 1
                    elif "too large" in reason:
                        self.stats.large_files += 1
                    else:
                        self.stats.error_files += 1

                    if verbose:
                        print(f"  Skipping {rel_file_path}: {reason}")

        # Calculate processing time
        end_time = datetime.now()
        self.stats.processing_time = (end_time - start_time).total_seconds()

        if verbose:
            print(f"Scan complete: {len(all_files)} files to process")

        return sorted(all_files), self.stats

    def generate_header(self, files: List[Path]) -> str:
        """
        Generate header section for the bundle.

        Args:
            files: List of files being processed

        Returns:
            Header string
        """
        header_lines = [
            "=" * 80,
            "PROJECT BUNDLE GENERATED BY PROJPACK",
            "=" * 80,
            "",
            f"Project Root: {self.project_root}",
            f"Generated: {datetime.now().strftime('%Y-%m-%d %H:%M:%S')}",
            f"Total Files: {len(files)}",
            ""
        ]

        if self.include_stats:
            header_lines.extend([
                "STATISTICS:",
                f"  Files processed: {len(files)}",
                f"  Total size: {self.file_handler.format_file_size(self.stats.total_size)}",
                f"  Files ignored: {self.stats.ignored_files}",
                f"  Binary files skipped: {self.stats.binary_files}",
                f"  Large files skipped: {self.stats.large_files}",
                f"  Directories scanned: {self.stats.directories_scanned}",
                f"  Processing time: {self.stats.processing_time:.2f}s",
                ""
            ])

        header_lines.extend([
            "FILE STRUCTURE:",
            self._generate_tree_view(files),
            "",
            "=" * 80,
            "FILE CONTENTS",
            "=" * 80,
            ""
        ])

        return "\n".join(header_lines)

    def _generate_tree_view(self, files: List[Path]) -> str:
        """
        Generate a tree-like view of the file structure.

        Args:
            files: List of files to include in tree

        Returns:
            Tree view string
        """
        # Build directory structure
        tree_structure = {}

        for file_path in files:
            rel_path = file_path.relative_to(self.project_root)
            parts = rel_path.parts

            current = tree_structure
            for part in parts[:-1]:  # Directories
                if part not in current:
                    current[part] = {}
                current = current[part]

            # Add file
            current[parts[-1]] = None

        return self._format_tree(tree_structure, "", True)

    def _format_tree(self, structure: Dict, prefix: str = "", is_last: bool = True) -> str:
        """
        Format directory structure as tree view.

        Args:
            structure: Nested dictionary representing directory structure
            prefix: Current line prefix for indentation
            is_last: Whether this is the last item in its parent

        Returns:
            Formatted tree string
        """
        lines = []
        items = list(structure.items())

        for i, (name, children) in enumerate(items):
            is_last_item = i == len(items) - 1

            # Current item
            connector = "└── " if is_last_item else "├── "
            lines.append(f"{prefix}{connector}{name}")

            # Children (if it's a directory)
            if children is not None:
                extension = "    " if is_last_item else "│   "
                child_tree = self._format_tree(children, prefix + extension, is_last_item)
                lines.append(child_tree)

        return "\n".join(lines)

    def format_file_content(self, file_path: Path) -> str:
        """
        Format a single file's content with header and metadata.

        Args:
            file_path: Path to the file

        Returns:
            Formatted file content string
        """
        rel_path = file_path.relative_to(self.project_root)

        # Read file content
        content, encoding = self.file_handler.read_file_content(file_path)
        file_size = self.file_handler.get_file_size(file_path)

        # Create file header
        separator = "-" * 60
        header_lines = [
            f"FILE: {rel_path}",
            separator,
            f"Size: {self.file_handler.format_file_size(file_size)}",
            f"Encoding: {encoding}",
            separator,
            ""
        ]

        header = "\n".join(header_lines)

        # Add content with proper separation
        formatted_content = f"{header}{content}\n\n"

        return formatted_content

    def pack_project(self,
                    output_file: Optional[str] = None,
                    verbose: bool = False) -> str:
        """
        Pack the entire project into a single bundle.

        Args:
            output_file: Output file path. If None, returns content as string
            verbose: Print detailed progress information

        Returns:
            Bundle content as string
        """
        if verbose:
            print("Starting project packing...")

        # Scan project
        files, stats = self.scan_project(verbose)

        if not files:
            message = "No files found to process!"
            if verbose:
                print(message)
            return message

        # Generate bundle
        bundle_parts = [self.generate_header(files)]

        # Process each file
        for i, file_path in enumerate(files, 1):
            if verbose:
                rel_path = file_path.relative_to(self.project_root)
                print(f"Processing ({i}/{len(files)}): {rel_path}")

            try:
                file_content = self.format_file_content(file_path)
                bundle_parts.append(file_content)
            except Exception as e:
                error_content = f"FILE: {file_path.relative_to(self.project_root)}\n"
                error_content += f"ERROR: Could not process file - {e}\n\n"
                bundle_parts.append(error_content)
                self.stats.error_files += 1

        # Add footer
        footer = [
            "=" * 80,
            "END OF PROJECT BUNDLE",
            "=" * 80
        ]
        bundle_parts.append("\n".join(footer))

        # Combine all parts
        bundle_content = "\n".join(bundle_parts)

        # Save to file if specified
        if output_file:
            output_path = Path(output_file)
            try:
                with open(output_path, 'w', encoding='utf-8') as f:
                    f.write(bundle_content)

                if verbose:
                    bundle_size = len(bundle_content.encode('utf-8'))
                    print(f"Bundle saved to: {output_path}")
                    print(f"Bundle size: {self.file_handler.format_file_size(bundle_size)}")

            except Exception as e:
                error_msg = f"Error saving bundle to {output_path}: {e}"
                if verbose:
                    print(error_msg)
                raise IOError(error_msg)

        return bundle_content

    def get_project_summary(self) -> Dict:
        """
        Get a summary of the project analysis.

        Returns:
            Dictionary with project summary information
        """
        return {
            'project_root': str(self.project_root),
            'total_files': self.stats.total_files,
            'processed_files': self.stats.total_files - self.stats.ignored_files -
                              self.stats.binary_files - self.stats.large_files - self.stats.error_files,
            'ignored_files': self.stats.ignored_files,
            'binary_files': self.stats.binary_files,
            'large_files': self.stats.large_files,
            'error_files': self.stats.error_files,
            'total_size': self.stats.total_size,
            'total_size_formatted': self.file_handler.format_file_size(self.stats.total_size),
            'directories_scanned': self.stats.directories_scanned,
            'processing_time': self.stats.processing_time
        }


if __name__ == "__main__":
    # Quick test of the project packer
    packer = ProjectPacker(".", include_stats=True)

    print("Testing project packer...")
    summary = packer.get_project_summary()

    # Scan only (don't generate full bundle for test)
    files, stats = packer.scan_project(verbose=True)

    print(f"\nProject Summary:")
    print(f"  Root: {summary['project_root']}")
    print(f"  Files found: {len(files)}")
    print(f"  Total size: {summary['total_size_formatted']}")