"""
ProjPack File Handler

Handles file reading, binary detection, encoding, and size validation.
Works with IgnoreParser to filter files appropriately.
"""

import os
import mimetypes
from pathlib import Path
from typing import Optional, Tuple, List, Set
import chardet


class FileHandler:
    """Handles file operations for ProjPack."""

    def __init__(self, max_file_size: int = 10 * 1024 * 1024):  # 10MB default
        """
        Initialize the file handler.

        Args:
            max_file_size: Maximum file size in bytes to process (default: 10MB)
        """
        self.max_file_size = max_file_size

        # Common text file extensions
        self.text_extensions = {
            '.txt', '.md', '.rst', '.py', '.js', '.ts', '.jsx', '.tsx',
            '.html', '.htm', '.css', '.scss', '.sass', '.less',
            '.json', '.xml', '.yaml', '.yml', '.toml', '.ini', '.cfg',
            '.sql', '.sh', '.bash', '.bat', '.ps1',
            '.c', '.cpp', '.h', '.hpp', '.java', '.cs', '.php',
            '.rb', '.go', '.rs', '.swift', '.kt', '.scala',
            '.r', '.m', '.pl', '.lua', '.vim',
            '.dockerfile', '.gitignore', '.gitattributes',
            '.env', '.env.example', '.editorconfig',
            '.log', '.conf', '.config'
        }

        # Known binary extensions to skip
        self.binary_extensions = {
            '.exe', '.dll', '.so', '.dylib', '.bin',
            '.jpg', '.jpeg', '.png', '.gif', '.bmp', '.ico', '.svg',
            '.mp4', '.avi', '.mov', '.wmv', '.flv', '.webm',
            '.mp3', '.wav', '.flac', '.aac', '.ogg',
            '.zip', '.tar', '.gz', '.bz2', '.xz', '.7z', '.rar',
            '.pdf', '.doc', '.docx', '.xls', '.xlsx', '.ppt', '.pptx',
            '.sqlite', '.db', '.mdb',
            '.ttf', '.otf', '.woff', '.woff2', '.eot',
            '.pyc', '.pyo', '.pyd', '.class', '.jar'
        }

    def is_text_file(self, file_path: Path) -> bool:
        """
        Determine if a file is likely a text file.

        Args:
            file_path: Path to the file

        Returns:
            True if the file appears to be text
        """
        # Check extension first
        extension = file_path.suffix.lower()

        if extension in self.binary_extensions:
            return False

        if extension in self.text_extensions:
            return True

        # Check MIME type
        mime_type, _ = mimetypes.guess_type(str(file_path))
        if mime_type:
            if mime_type.startswith('text/'):
                return True
            if mime_type in ['application/json', 'application/xml', 'application/javascript']:
                return True

        # If no extension or unknown, check file content
        return self._is_text_content(file_path)

    def _is_text_content(self, file_path: Path) -> bool:
        """
        Check if file content appears to be text by examining first few bytes.

        Args:
            file_path: Path to the file

        Returns:
            True if content appears to be text
        """
        try:
            with open(file_path, 'rb') as f:
                # Read first 1KB to check for binary content
                chunk = f.read(1024)

                if not chunk:
                    return True  # Empty file, treat as text

                # Check for null bytes (common in binary files)
                if b'\x00' in chunk:
                    return False

                # Check for high ratio of printable characters
                printable_ratio = sum(1 for byte in chunk if 32 <= byte <= 126 or byte in [9, 10, 13]) / len(chunk)
                return printable_ratio > 0.7

        except (IOError, OSError):
            return False

    def get_file_size(self, file_path: Path) -> int:
        """
        Get file size in bytes.

        Args:
            file_path: Path to the file

        Returns:
            File size in bytes, or -1 if error
        """
        try:
            return file_path.stat().st_size
        except (OSError, IOError):
            return -1

    def is_file_too_large(self, file_path: Path) -> bool:
        """
        Check if file exceeds size limit.

        Args:
            file_path: Path to the file

        Returns:
            True if file is too large
        """
        size = self.get_file_size(file_path)
        return size > self.max_file_size if size >= 0 else True

    def detect_encoding(self, file_path: Path) -> str:
        """
        Detect file encoding using chardet.

        Args:
            file_path: Path to the file

        Returns:
            Detected encoding or 'utf-8' as fallback
        """
        try:
            with open(file_path, 'rb') as f:
                raw_data = f.read(min(10000, self.max_file_size))  # Sample first 10KB

            result = chardet.detect(raw_data)
            encoding = result.get('encoding', 'utf-8')
            confidence = result.get('confidence', 0)

            # Use utf-8 if confidence is too low
            if confidence < 0.7:
                encoding = 'utf-8'

            return encoding or 'utf-8'

        except Exception:
            return 'utf-8'

    def read_file_content(self, file_path: Path, encoding: Optional[str] = None) -> Tuple[str, str]:
        """
        Read file content with proper encoding handling.

        Args:
            file_path: Path to the file
            encoding: Specific encoding to use, or None for auto-detection

        Returns:
            Tuple of (content, encoding_used)
        """
        if encoding is None:
            encoding = self.detect_encoding(file_path)

        try:
            with open(file_path, 'r', encoding=encoding, errors='replace') as f:
                content = f.read()
            return content, encoding

        except UnicodeDecodeError:
            # Fallback to utf-8 with error replacement
            try:
                with open(file_path, 'r', encoding='utf-8', errors='replace') as f:
                    content = f.read()
                return content, 'utf-8'
            except Exception as e:
                return f"[Error reading file: {e}]", 'utf-8'

        except Exception as e:
            return f"[Error reading file: {e}]", encoding

    def should_process_file(self, file_path: Path) -> Tuple[bool, str]:
        """
        Determine if a file should be processed based on various criteria.

        Args:
            file_path: Path to the file

        Returns:
            Tuple of (should_process, reason)
        """
        # Check if file exists
        if not file_path.exists():
            return False, "File does not exist"

        if not file_path.is_file():
            return False, "Not a regular file"

        # Check file size
        if self.is_file_too_large(file_path):
            size_mb = self.get_file_size(file_path) / (1024 * 1024)
            return False, f"File too large ({size_mb:.1f}MB > {self.max_file_size / (1024 * 1024)}MB)"

        # Check if it's a text file
        if not self.is_text_file(file_path):
            return False, "Binary file"

        return True, "OK"

    def get_file_info(self, file_path: Path) -> dict:
        """
        Get comprehensive information about a file.

        Args:
            file_path: Path to the file

        Returns:
            Dictionary with file information
        """
        info = {
            'path': str(file_path),
            'name': file_path.name,
            'extension': file_path.suffix,
            'size': self.get_file_size(file_path),
            'is_text': self.is_text_file(file_path),
            'encoding': None,
            'should_process': False,
            'reason': 'Not checked'
        }

        should_process, reason = self.should_process_file(file_path)
        info['should_process'] = should_process
        info['reason'] = reason

        if should_process:
            info['encoding'] = self.detect_encoding(file_path)

        return info

    def collect_files(self, root_path: Path, ignore_parser=None) -> List[Path]:
        """
        Collect all files in directory tree that should be processed.

        Args:
            root_path: Root directory to scan
            ignore_parser: IgnoreParser instance for filtering

        Returns:
            List of file paths to process
        """
        collected_files = []

        for root, dirs, files in os.walk(root_path):
            root_path_obj = Path(root)

            # Get relative path for ignore checking
            try:
                rel_root = root_path_obj.relative_to(root_path)
                rel_root_str = str(rel_root) if str(rel_root) != '.' else ''
            except ValueError:
                continue

            # Filter directories using ignore parser
            if ignore_parser:
                dirs_to_remove = []
                for d in dirs[:]:  # Copy list to modify during iteration
                    dir_path = os.path.join(rel_root_str, d) if rel_root_str else d
                    if ignore_parser.should_ignore(dir_path, is_directory=True):
                        dirs_to_remove.append(d)

                for d in dirs_to_remove:
                    dirs.remove(d)

            # Process files
            for f in files:
                file_path = root_path_obj / f
                rel_file_path = os.path.join(rel_root_str, f) if rel_root_str else f

                # Check ignore patterns
                if ignore_parser and ignore_parser.should_ignore(rel_file_path, is_directory=False):
                    continue

                # Check if file should be processed
                should_process, _ = self.should_process_file(file_path)
                if should_process:
                    collected_files.append(file_path)

        return sorted(collected_files)

    def format_file_size(self, size_bytes: int) -> str:
        """
        Format file size in human-readable format.

        Args:
            size_bytes: Size in bytes

        Returns:
            Formatted size string
        """
        if size_bytes < 1024:
            return f"{size_bytes} B"
        elif size_bytes < 1024 * 1024:
            return f"{size_bytes / 1024:.1f} KB"
        elif size_bytes < 1024 * 1024 * 1024:
            return f"{size_bytes / (1024 * 1024):.1f} MB"
        else:
            return f"{size_bytes / (1024 * 1024 * 1024):.1f} GB"


if __name__ == "__main__":
    # Quick test of the file handler
    handler = FileHandler()

    test_files = [
        "main.py",
        "README.md",
        "image.jpg",
        "data.json"
    ]

    print("Testing file handler:")
    for file_name in test_files:
        file_path = Path(file_name)
        if file_path.exists():
            info = handler.get_file_info(file_path)
            print(f"  {file_name}:")
            print(f"    Size: {handler.format_file_size(info['size'])}")
            print(f"    Text file: {info['is_text']}")
            print(f"    Should process: {info['should_process']} ({info['reason']})")
        else:
            print(f"  {file_name}: File not found")