"""
ProjPack Ignore Parser

Handles parsing of .ppignore files with gitignore-like syntax.
Supports glob patterns, negation, and directory-specific rules.
"""

import os
import re
from pathlib import Path
from typing import List, Set, Optional


class IgnoreParser:
    """Parser for .ppignore files with gitignore-like syntax."""

    def __init__(self, ignore_file_path: Optional[str] = None, project_root: Optional[str] = None):
        """
        Initialize the ignore parser.

        Args:
            ignore_file_path: Path to .ppignore file. If None, looks for .ppignore in project_root
            project_root: Root directory of the project. If None, uses current directory
        """
        self.project_root = Path(project_root or os.getcwd()).resolve()
        self.ignore_patterns: List[str] = []
        self.compiled_patterns: List[tuple] = []  # (pattern, is_negation, is_directory_only)

        # Default patterns that are commonly ignored
        self.default_patterns = [
            '__pycache__/',
            '*.pyc',
            '*.pyo',
            '*.pyd',
            '.Python',
            'build/',
            'develop-eggs/',
            'dist/',
            'downloads/',
            'eggs/',
            '.eggs/',
            'lib/',
            'lib64/',
            'parts/',
            'sdist/',
            'var/',
            'wheels/',
            '*.egg-info/',
            '.installed.cfg',
            '*.egg',
            '.env',
            '.venv',
            'env/',
            'venv/',
            'ENV/',
            'env.bak/',
            'venv.bak/',
            '.DS_Store',
            'Thumbs.db',
            '*.swp',
            '*.swo',
            '*~',
            '.git/',
            '.svn/',
            '.hg/',
            '.bzr/',
            'node_modules/',
            '.npm',
            'bower_components/',
            'coverage/',
            '.coverage',
            '.pytest_cache/',
            '.tox/',
            '.cache/',
            'nosetests.xml',
            'coverage.xml',
            '*.cover',
            '.hypothesis/',
        ]

        # Try to find and load ignore file
        if ignore_file_path:
            self.ignore_file_path = Path(ignore_file_path)
        else:
            self.ignore_file_path = self.project_root / '.ppignore'

        self.load_patterns()

    def load_patterns(self):
        """Load patterns from .ppignore file and compile them."""
        self.ignore_patterns = self.default_patterns.copy()

        if self.ignore_file_path.exists():
            try:
                with open(self.ignore_file_path, 'r', encoding='utf-8') as f:
                    for line in f:
                        line = line.strip()
                        if line and not line.startswith('#'):
                            self.ignore_patterns.append(line)
            except (IOError, UnicodeDecodeError) as e:
                print(f"Warning: Could not read {self.ignore_file_path}: {e}")

        self._compile_patterns()

    def _compile_patterns(self):
        """Compile patterns into regex for efficient matching."""
        self.compiled_patterns = []

        for pattern in self.ignore_patterns:
            is_negation = pattern.startswith('!')
            if is_negation:
                pattern = pattern[1:]

            is_directory_only = pattern.endswith('/')
            if is_directory_only:
                pattern = pattern[:-1]

            # Convert glob pattern to regex
            regex_pattern = self._glob_to_regex(pattern)

            try:
                compiled = re.compile(regex_pattern, re.IGNORECASE if os.name == 'nt' else 0)
                self.compiled_patterns.append((compiled, is_negation, is_directory_only))
            except re.error as e:
                print(f"Warning: Invalid pattern '{pattern}': {e}")

    def _glob_to_regex(self, pattern: str) -> str:
        """Convert glob pattern to regex."""
        # Escape special regex characters except glob wildcards
        pattern = re.escape(pattern)

        # Replace escaped glob patterns with regex equivalents
        pattern = pattern.replace(r'\*\*', '.*')  # ** matches any number of directories
        pattern = pattern.replace(r'\*', '[^/]*')  # * matches anything except directory separator
        pattern = pattern.replace(r'\?', '[^/]')  # ? matches single character except directory separator

        # Handle leading slash (absolute path from project root)
        if pattern.startswith(r'\/'):
            pattern = pattern[2:]  # Remove escaped leading slash
            return f'^{pattern}$'

        # Handle patterns that should match anywhere in the path
        return f'(^|/)({pattern})(/|$)'

    def should_ignore(self, file_path: str, is_directory: bool = False) -> bool:
        """
        Check if a file or directory should be ignored.

        Args:
            file_path: Relative path from project root
            is_directory: Whether the path is a directory

        Returns:
            True if the file/directory should be ignored
        """
        # Normalize path separators
        normalized_path = file_path.replace('\\', '/')

        # Start with ignore by default for negation patterns
        should_ignore_result = False

        for compiled_pattern, is_negation, is_directory_only in self.compiled_patterns:
            # Skip directory-only patterns for files
            if is_directory_only and not is_directory:
                continue

            if compiled_pattern.search(normalized_path):
                if is_negation:
                    should_ignore_result = False
                else:
                    should_ignore_result = True

        return should_ignore_result

    def get_ignored_paths(self, root_path: Optional[str] = None) -> Set[str]:
        """
        Get all paths that would be ignored in the given directory.

        Args:
            root_path: Directory to scan. If None, uses project_root

        Returns:
            Set of relative paths that would be ignored
        """
        if root_path is None:
            root_path = self.project_root
        else:
            root_path = Path(root_path)

        ignored_paths = set()

        for root, dirs, files in os.walk(root_path):
            root_path_obj = Path(root)

            # Get relative path from project root
            try:
                rel_root = root_path_obj.relative_to(self.project_root)
                rel_root_str = str(rel_root) if str(rel_root) != '.' else ''
            except ValueError:
                # Path is outside project root
                continue

            # Check directories
            dirs_to_remove = []
            for d in dirs:
                dir_path = os.path.join(rel_root_str, d) if rel_root_str else d
                if self.should_ignore(dir_path, is_directory=True):
                    ignored_paths.add(dir_path)
                    dirs_to_remove.append(d)

            # Remove ignored directories from traversal
            for d in dirs_to_remove:
                dirs.remove(d)

            # Check files
            for f in files:
                file_path = os.path.join(rel_root_str, f) if rel_root_str else f
                if self.should_ignore(file_path, is_directory=False):
                    ignored_paths.add(file_path)

        return ignored_paths

    def add_pattern(self, pattern: str):
        """Add a new ignore pattern."""
        self.ignore_patterns.append(pattern)
        self._compile_patterns()

    def remove_pattern(self, pattern: str):
        """Remove an ignore pattern."""
        if pattern in self.ignore_patterns:
            self.ignore_patterns.remove(pattern)
            self._compile_patterns()

    def save_ignore_file(self, patterns: Optional[List[str]] = None):
        """
        Save patterns to .ppignore file.

        Args:
            patterns: Patterns to save. If None, saves current patterns excluding defaults
        """
        if patterns is None:
            # Save only non-default patterns
            patterns = [p for p in self.ignore_patterns if p not in self.default_patterns]

        try:
            with open(self.ignore_file_path, 'w', encoding='utf-8') as f:
                f.write("# ProjPack ignore file\n")
                f.write("# Syntax similar to .gitignore\n\n")
                for pattern in patterns:
                    f.write(f"{pattern}\n")
        except IOError as e:
            raise IOError(f"Could not write to {self.ignore_file_path}: {e}")


def create_default_ppignore(project_path: str = '.'):
    """Create a default .ppignore file in the specified directory."""
    parser = IgnoreParser(project_root=project_path)
    ignore_file = Path(project_path) / '.ppignore'

    if ignore_file.exists():
        response = input(f"{ignore_file} already exists. Overwrite? (y/N): ")
        if response.lower() != 'y':
            return

    try:
        parser.save_ignore_file(parser.default_patterns)
        print(f"Created default .ppignore file at {ignore_file}")
    except IOError as e:
        print(f"Error creating .ppignore file: {e}")


if __name__ == "__main__":
    # Quick test of the ignore parser
    parser = IgnoreParser()

    test_paths = [
        "main.py",
        "__pycache__/test.pyc",
        "node_modules/package/index.js",
        "src/main.py",
        "build/output.exe",
        ".git/config",
        "requirements.txt"
    ]

    print("Testing ignore patterns:")
    for path in test_paths:
        ignored = parser.should_ignore(path,
                                       is_directory="/" in path and not path.endswith(('.py', '.txt', '.js', '.exe')))
        print(f"  {path}: {'IGNORED' if ignored else 'INCLUDED'}")